require "fileutils"

Dir.glob("Vitis_HLS/*/include/mpfr.h").each do |fn|
  # check version
  next if fn !~ /20(\d\d\.\d)/
  version = $1
  print "#{version} - "
  if version.to_f >= 22.2
    puts "not required"
    next
  end
  # check already patched
  patched = false
  open(fn) do |file|
    while line = file.gets
      if line =~ /fix for GMP issue/
        patched = true
        break
      end
    end
  end
  if patched
    puts "already patched"
    next
  end

  # patch the file
  FileUtils.cp(fn, fn + ".bak")
  open(fn + ".bak") do |file_in|
    open(fn, "w") do |file_out|
      while line = file_in.gets
        line.chomp!
        if line =~ /#ifndef __GMP_H__/
          file_in.gets # discard one line
          file_out.puts '/* fix for GMP issue Ref: https://github.com/Xilinx/finn-hlslib/issues/34 */'
          file_out.puts '#ifndef __GMP_H__'
          file_out.puts '  #include <gmp.h>'
          file_out.puts '#else'
          file_out.puts '  #include "' + __dir__ + '/Vivado/20' + version + '/include/gmp.h"'
          file_out.puts '  #ifndef __gmp_const'
          file_out.puts '    #define __gmp_const const'
          file_out.puts '  #endif'
          patched = true
        else
          file_out.puts line
        end
      end
    end
  end
  puts "patched"     if   patched
  puts "not patched" if ! patched
end